@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -35,40 +35,38 @@ class CControllerAuthenticationUpdate extends CController {
 
 	protected function checkInput() {
 		$fields = [
-			'form_refresh' =>				'string',
-			'ldap_test_user' =>				'string',
-			'ldap_test_password' =>			'string',
-			'ldap_test' =>					'in 1',
-			'db_authentication_type' =>		'int32',
-			'change_bind_password' =>		'in 0,1',
-			'authentication_type' =>		'in '.ZBX_AUTH_INTERNAL.','.ZBX_AUTH_LDAP,
-			'http_case_sensitive' =>		'in '.ZBX_AUTH_CASE_INSENSITIVE.','.ZBX_AUTH_CASE_SENSITIVE,
-			'ldap_case_sensitive' =>		'in '.ZBX_AUTH_CASE_INSENSITIVE.','.ZBX_AUTH_CASE_SENSITIVE,
-			'ldap_configured' =>			'in '.ZBX_AUTH_LDAP_DISABLED.','.ZBX_AUTH_LDAP_ENABLED,
-			'ldap_host' =>					'db config.ldap_host',
-			'ldap_port' =>					'int32',
-			'ldap_base_dn' =>				'db config.ldap_base_dn',
-			'ldap_bind_dn' =>				'db config.ldap_bind_dn',
-			'ldap_search_attribute' =>		'db config.ldap_search_attribute',
-			'ldap_bind_password' =>			'db config.ldap_bind_password',
-			'http_auth_enabled' =>			'in '.ZBX_AUTH_HTTP_DISABLED.','.ZBX_AUTH_HTTP_ENABLED,
-			'http_login_form' =>			'in '.ZBX_AUTH_FORM_ZABBIX.','.ZBX_AUTH_FORM_HTTP,
-			'http_strip_domains' =>			'db config.http_strip_domains',
-			'saml_auth_enabled' =>			'in '.ZBX_AUTH_SAML_DISABLED.','.ZBX_AUTH_SAML_ENABLED,
-			'saml_idp_entityid' =>			'db config.saml_idp_entityid',
-			'saml_sso_url' =>				'db config.saml_sso_url',
-			'saml_slo_url' =>				'db config.saml_slo_url',
-			'saml_username_attribute' =>	'db config.saml_username_attribute',
-			'saml_sp_entityid' =>			'db config.saml_sp_entityid',
-			'saml_nameid_format' =>			'db config.saml_nameid_format',
-			'saml_sign_messages' =>			'in 0,1',
-			'saml_sign_assertions' =>		'in 0,1',
-			'saml_sign_authn_requests' =>	'in 0,1',
-			'saml_sign_logout_requests' =>	'in 0,1',
-			'saml_sign_logout_responses' =>	'in 0,1',
-			'saml_encrypt_nameid' =>		'in 0,1',
-			'saml_encrypt_assertions' =>	'in 0,1',
-			'saml_case_sensitive' =>		'in '.ZBX_AUTH_CASE_INSENSITIVE.','.ZBX_AUTH_CASE_SENSITIVE
+			'form_refresh' =>					'string',
+			'ldap_test_user' =>					'string',
+			'ldap_test_password' =>				'string',
+			'ldap_test' =>						'in 1',
+			'change_bind_password' =>			'in 0,1',
+			'authentication_type' =>			'in '.ZBX_AUTH_INTERNAL.','.ZBX_AUTH_LDAP,
+			'http_case_sensitive' =>			'in '.ZBX_AUTH_CASE_INSENSITIVE.','.ZBX_AUTH_CASE_SENSITIVE,
+			'ldap_configured' =>				'in '.ZBX_AUTH_LDAP_DISABLED.','.ZBX_AUTH_LDAP_ENABLED,
+			'ldap_servers' =>					'array',
+			'ldap_default_row_index' =>			'int32',
+			'ldap_case_sensitive' =>			'in '.ZBX_AUTH_CASE_INSENSITIVE.','.ZBX_AUTH_CASE_SENSITIVE,
+			'ldap_removed_userdirectoryids' =>	'array',
+			'http_auth_enabled' =>				'in '.ZBX_AUTH_HTTP_DISABLED.','.ZBX_AUTH_HTTP_ENABLED,
+			'http_login_form' =>				'in '.ZBX_AUTH_FORM_ZABBIX.','.ZBX_AUTH_FORM_HTTP,
+			'http_strip_domains' =>				'db config.http_strip_domains',
+			'saml_auth_enabled' =>				'in '.ZBX_AUTH_SAML_DISABLED.','.ZBX_AUTH_SAML_ENABLED,
+			'saml_idp_entityid' =>				'db config.saml_idp_entityid',
+			'saml_sso_url' =>					'db config.saml_sso_url',
+			'saml_slo_url' =>					'db config.saml_slo_url',
+			'saml_username_attribute' =>		'db config.saml_username_attribute',
+			'saml_sp_entityid' =>				'db config.saml_sp_entityid',
+			'saml_nameid_format' =>				'db config.saml_nameid_format',
+			'saml_sign_messages' =>				'in 0,1',
+			'saml_sign_assertions' =>			'in 0,1',
+			'saml_sign_authn_requests' =>		'in 0,1',
+			'saml_sign_logout_requests' =>		'in 0,1',
+			'saml_sign_logout_responses' =>		'in 0,1',
+			'saml_encrypt_nameid' =>			'in 0,1',
+			'saml_encrypt_assertions' =>		'in 0,1',
+			'saml_case_sensitive' =>			'in '.ZBX_AUTH_CASE_INSENSITIVE.','.ZBX_AUTH_CASE_SENSITIVE,
+			'passwd_min_length' =>				'int32',
+			'passwd_check_rules' =>				'array'
 		];
 
 		$ret = $this->validateInput($fields);
@@ -111,78 +109,33 @@ class CControllerAuthenticationUpdate extends CController {
 	 *
 	 * @return bool
 	 */
-	private function validateLdap() {
-		$is_valid = true;
-		$ldap_status = (new CFrontendSetup())->checkPhpLdapModule();
-		$ldap_fields = ['ldap_host', 'ldap_port', 'ldap_base_dn', 'ldap_search_attribute', 'ldap_configured'];
-		$ldap_auth_original = [
-			'ldap_host' => CAuthenticationHelper::get(CAuthenticationHelper::LDAP_HOST),
-			'ldap_port' => CAuthenticationHelper::get(CAuthenticationHelper::LDAP_PORT),
-			'ldap_base_dn' => CAuthenticationHelper::get(CAuthenticationHelper::LDAP_BASE_DN),
-			'ldap_search_attribute' => CAuthenticationHelper::get(CAuthenticationHelper::LDAP_SEARCH_ATTRIBUTE),
-			'ldap_configured' => CAuthenticationHelper::get(CAuthenticationHelper::LDAP_CONFIGURED),
-			'ldap_bind_dn' => CAuthenticationHelper::get(CAuthenticationHelper::LDAP_BIND_DN),
-			'ldap_bind_password' => CAuthenticationHelper::get(CAuthenticationHelper::LDAP_BIND_PASSWORD)
-		];
-		$ldap_auth = $ldap_auth_original;
-		$this->getInputs($ldap_auth, array_merge($ldap_fields, ['ldap_bind_dn', 'ldap_bind_password']));
-		$ldap_auth_changed = array_diff_assoc($ldap_auth, $ldap_auth_original);
+	private function validateLdap(): bool {
+		$ldap_enabled = $this->getInput('ldap_configured', ZBX_AUTH_LDAP_DISABLED) == ZBX_AUTH_LDAP_ENABLED;
+		$ldap_servers = $this->getInput('ldap_servers', []);
 
-		if (!$ldap_auth_changed && !$this->hasInput('ldap_test')) {
-			return $is_valid;
-		}
+		if ($ldap_enabled) {
+			$ldap_status = (new CFrontendSetup())->checkPhpLdapModule();
 
-		if ($this->getInput('ldap_bind_password', '') !== '') {
-			$ldap_fields[] = 'ldap_bind_dn';
-		}
+			if ($ldap_status['result'] != CFrontendSetup::CHECK_OK) {
+				CMessageHelper::setErrorTitle($ldap_status['error']);
+				return false;
+			}
 
-		foreach ($ldap_fields as $field) {
-			if (trim($ldap_auth[$field]) === '') {
-				CMessageHelper::setErrorTitle(_s('Incorrect value for field "%1$s": %2$s.', $field, _('cannot be empty')));
-				$is_valid = false;
-				break;
+			if (!$ldap_servers) {
+				CMessageHelper::setErrorTitle(_('At least one LDAP server must exist.'));
+				return false;
 			}
 		}
 
-		if ($is_valid
-				&& ($ldap_auth['ldap_port'] < ZBX_MIN_PORT_NUMBER
-					|| $ldap_auth['ldap_port'] > ZBX_MAX_PORT_NUMBER)) {
-			CMessageHelper::setErrorTitle(_s(
-				'Incorrect value "%1$s" for "%2$s" field: must be between %3$s and %4$s.', $this->getInput('ldap_port'),
-				'ldap_port', ZBX_MIN_PORT_NUMBER, ZBX_MAX_PORT_NUMBER
-			));
-			$is_valid = false;
+		if ($ldap_servers
+				&& (!$this->hasInput('ldap_default_row_index')
+					|| !array_key_exists($this->getInput('ldap_default_row_index'), $ldap_servers)
+				)) {
+			CMessageHelper::setErrorTitle(_('Default LDAP server must be specified.'));
+			return false;
 		}
 
-		if ($ldap_status['result'] != CFrontendSetup::CHECK_OK) {
-			CMessageHelper::setErrorTitle($ldap_status['error']);
-			$is_valid = false;
-		}
-		elseif ($is_valid) {
-			$ldap_validator = new CLdapAuthValidator([
-				'conf' => [
-					'host' => $ldap_auth['ldap_host'],
-					'port' => $ldap_auth['ldap_port'],
-					'base_dn' => $ldap_auth['ldap_base_dn'],
-					'bind_dn' => $ldap_auth['ldap_bind_dn'],
-					'bind_password' => $ldap_auth['ldap_bind_password'],
-					'search_attribute' => $ldap_auth['ldap_search_attribute']
-				],
-				'detailed_errors' => true
-			]);
-
-			$login = $ldap_validator->validate([
-				'user' => $this->getInput('ldap_test_user', CWebUser::$data['alias']),
-				'password' => $this->getInput('ldap_test_password', '')
-			]);
-
-			if (!$login) {
-				CMessageHelper::setErrorTitle($ldap_validator->getError());
-				$is_valid = false;
-			}
-		}
-
-		return $is_valid;
+		return true;
 	}
 
 	/**
@@ -228,13 +181,44 @@ class CControllerAuthenticationUpdate extends CController {
 		return $this->checkAccess(CRoleHelper::UI_ADMINISTRATION_AUTHENTICATION);
 	}
 
+	/**
+	 * In case of error, convert array back to integer (string) so edit form does not fail.
+	 *
+	 * @return array
+	 */
+	public function getInputAll() {
+		$input = parent::getInputAll();
+		$rules = $input['passwd_check_rules'];
+		$input['passwd_check_rules'] = 0x00;
+
+		foreach ($rules as $rule) {
+			$input['passwd_check_rules'] |= $rule;
+		}
+
+		// CNewValidator thinks int32 must be a string type integer.
+		$input['passwd_check_rules'] = (string) $input['passwd_check_rules'];
+
+		return $input;
+	}
+
 	protected function doAction() {
-		$auth_valid = ($this->getInput('ldap_configured', '') == ZBX_AUTH_LDAP_ENABLED)
-			? $this->validateLdap()
-			: $this->validateDefaultAuth();
+		$auth_valid = $this->validateDefaultAuth();
+
+		if ($auth_valid && !$this->validateLdap()) {
+			$auth_valid = false;
+		}
 
 		if ($auth_valid && $this->getInput('saml_auth_enabled', ZBX_AUTH_SAML_DISABLED) == ZBX_AUTH_SAML_ENABLED) {
-			$auth_valid &= $this->validateSamlAuth();
+			if (!$this->validateSamlAuth()) {
+				$auth_valid = false;
+			}
+		}
+
+		if ($auth_valid) {
+			[$auth_valid, $ldap_userdirectoryid] = $this->processLdapServers(
+				$this->getInput('ldap_servers', []),
+				$this->getInput('ldap_default_row_index', 0)
+			);
 		}
 
 		if (!$auth_valid) {
@@ -244,29 +228,15 @@ class CControllerAuthenticationUpdate extends CController {
 			return;
 		}
 
-		// Only ZBX_AUTH_LDAP have 'Test' option.
-		if ($this->hasInput('ldap_test')) {
-			CMessageHelper::setSuccessTitle(_('LDAP login successful'));
-			$this->response->setFormData($this->getInputAll());
-			$this->setResponse($this->response);
-
-			return;
-		}
-
 		$auth_params = [
 			CAuthenticationHelper::AUTHENTICATION_TYPE,
 			CAuthenticationHelper::HTTP_AUTH_ENABLED,
 			CAuthenticationHelper::HTTP_LOGIN_FORM,
 			CAuthenticationHelper::HTTP_STRIP_DOMAINS,
 			CAuthenticationHelper::HTTP_CASE_SENSITIVE,
-			CAuthenticationHelper::LDAP_CASE_SENSITIVE,
 			CAuthenticationHelper::LDAP_CONFIGURED,
-			CAuthenticationHelper::LDAP_HOST,
-			CAuthenticationHelper::LDAP_PORT,
-			CAuthenticationHelper::LDAP_BASE_DN,
-			CAuthenticationHelper::LDAP_BIND_DN,
-			CAuthenticationHelper::LDAP_SEARCH_ATTRIBUTE,
-			CAuthenticationHelper::LDAP_BIND_PASSWORD,
+			CAuthenticationHelper::LDAP_USERDIRECTORYID,
+			CAuthenticationHelper::LDAP_CASE_SENSITIVE,
 			CAuthenticationHelper::SAML_AUTH_ENABLED,
 			CAuthenticationHelper::SAML_IDP_ENTITYID,
 			CAuthenticationHelper::SAML_SSO_URL,
@@ -281,7 +251,9 @@ class CControllerAuthenticationUpdate extends CController {
 			CAuthenticationHelper::SAML_SIGN_LOGOUT_RESPONSES,
 			CAuthenticationHelper::SAML_ENCRYPT_NAMEID,
 			CAuthenticationHelper::SAML_ENCRYPT_ASSERTIONS,
-			CAuthenticationHelper::SAML_CASE_SENSITIVE
+			CAuthenticationHelper::SAML_CASE_SENSITIVE,
+			CAuthenticationHelper::PASSWD_MIN_LENGTH,
+			CAuthenticationHelper::PASSWD_CHECK_RULES
 		];
 		$auth = [];
 		foreach ($auth_params as $param) {
@@ -291,8 +263,12 @@ class CControllerAuthenticationUpdate extends CController {
 		$fields = [
 			'authentication_type' => ZBX_AUTH_INTERNAL,
 			'ldap_configured' => ZBX_AUTH_LDAP_DISABLED,
+			'ldap_userdirectoryid' => $ldap_userdirectoryid,
+			'ldap_case_sensitive' => ZBX_AUTH_CASE_INSENSITIVE,
 			'http_auth_enabled' => ZBX_AUTH_HTTP_DISABLED,
-			'saml_auth_enabled' => ZBX_AUTH_SAML_DISABLED
+			'saml_auth_enabled' => ZBX_AUTH_SAML_DISABLED,
+			'passwd_min_length' => DB::getDefault('config', 'passwd_min_length'),
+			'passwd_check_rules' => DB::getDefault('config', 'passwd_check_rules')
 		];
 
 		if ($this->getInput('http_auth_enabled', ZBX_AUTH_HTTP_DISABLED) == ZBX_AUTH_HTTP_ENABLED) {
@@ -303,24 +279,6 @@ class CControllerAuthenticationUpdate extends CController {
 			];
 		}
 
-		if ($this->getInput('ldap_configured', ZBX_AUTH_LDAP_DISABLED) == ZBX_AUTH_LDAP_ENABLED) {
-			$fields += [
-				'ldap_host' => '',
-				'ldap_port' => '',
-				'ldap_base_dn' => '',
-				'ldap_bind_dn' => '',
-				'ldap_search_attribute' => '',
-				'ldap_case_sensitive' => 0
-			];
-
-			if ($this->hasInput('ldap_bind_password')) {
-				$fields['ldap_bind_password'] = '';
-			}
-			else {
-				unset($auth[CAuthenticationHelper::LDAP_BIND_PASSWORD]);
-			}
-		}
-
 		if ($this->getInput('saml_auth_enabled', ZBX_AUTH_SAML_DISABLED) == ZBX_AUTH_SAML_ENABLED) {
 			$fields += [
 				'saml_idp_entityid' => '',
@@ -342,31 +300,101 @@ class CControllerAuthenticationUpdate extends CController {
 
 		$data = $fields + $auth;
 		$this->getInputs($data, array_keys($fields));
-		$data = array_diff_assoc($data, $auth);
 
-		if (array_key_exists('ldap_bind_dn', $data) && trim($data['ldap_bind_dn']) === '') {
-			$data['ldap_bind_password'] = '';
+		$rules = $data['passwd_check_rules'];
+		$data['passwd_check_rules'] = 0x00;
+
+		foreach ($rules as $rule) {
+			$data['passwd_check_rules'] |= $rule;
 		}
 
+		$data = array_diff_assoc($data, $auth);
+		$result = true;
+
 		if ($data) {
 			$result = API::Authentication()->update($data);
 
-			if ($result) {
-				if (array_key_exists('authentication_type', $data)) {
-					$this->invalidateSessions();
-				}
-
-				CMessageHelper::setSuccessTitle(_('Authentication settings updated'));
-			}
-			else {
-				$this->response->setFormData($this->getInputAll());
-				CMessageHelper::setErrorTitle(_('Cannot update authentication'));
+			if ($result && array_key_exists('authentication_type', $data)) {
+				$this->invalidateSessions();
 			}
 		}
 
+		if ($result) {
+			$del_userdirectoryids = $this->getInput('ldap_removed_userdirectoryids', []);
+
+			if ($del_userdirectoryids) {
+				$result = API::UserDirectory()->delete($del_userdirectoryids);
+			}
+		}
+
+		if ($result) {
+			CMessageHelper::setSuccessTitle(_('Authentication settings updated'));
+		}
+		else {
+			$this->response->setFormData($this->getInputAll());
+			CMessageHelper::setErrorTitle(_('Cannot update authentication'));
+		}
+
 		$this->setResponse($this->response);
 	}
 
+	/**
+	 * Updates existing LDAP servers, creates new ones, removes deleted ones.
+	 *
+	 * @param array $ldap_servers
+	 * @param int $default_row_index
+	 *
+	 * @return array
+	 */
+	private function processLdapServers(array $ldap_servers, int $default_row_index): array {
+		$ins_ldap_servers = [];
+		$upd_ldap_servers = [];
+
+		$ldap_userdirectoryid = 0;
+		$default_ldap_mapping_index = null;
+
+		foreach ($ldap_servers as $row_index => $ldap_server) {
+			if (array_key_exists('userdirectoryid', $ldap_server)) {
+				$upd_ldap_servers[] = $ldap_server;
+
+				if ($default_row_index == $row_index) {
+					$ldap_userdirectoryid = $ldap_server['userdirectoryid'];
+				}
+			}
+			else {
+				$ins_ldap_servers[] = $ldap_server;
+
+				if ($default_row_index == $row_index) {
+					$default_ldap_mapping_index = count($ins_ldap_servers) - 1;
+				}
+			}
+		}
+
+		if ($upd_ldap_servers && !API::UserDirectory()->update($upd_ldap_servers)) {
+			return [false, $ldap_userdirectoryid];
+		}
+
+		if ($ins_ldap_servers) {
+			$inserted_ids = API::UserDirectory()->create($ins_ldap_servers);
+
+			if (!$inserted_ids) {
+				return [false, $ldap_userdirectoryid];
+			}
+
+			if ($default_ldap_mapping_index !== null) {
+				$ldap_userdirectoryid = $inserted_ids['userdirectoryids'][$default_ldap_mapping_index];
+			}
+		}
+
+		if ($ldap_servers && $ldap_userdirectoryid === 0) {
+			CMessageHelper::setErrorTitle(_('Failed to select default LDAP server.'));
+
+			return [false, $ldap_userdirectoryid];
+		}
+
+		return [true, $ldap_userdirectoryid];
+	}
+
 	/**
 	 * Mark all active GROUP_GUI_ACCESS_INTERNAL sessions, except current user sessions, as ZBX_SESSION_PASSIVE.
 	 *
