@@ -134,6 +134,9 @@ abstract class CGraphGeneral extends CApiService {
 		$this->updateReal($graphs);
 		$this->inherit($graphs);
 
+		$resource = ($this instanceof CGraph) ? CAudit::RESOURCE_GRAPH : CAudit::RESOURCE_GRAPH_PROTOTYPE;
+		$this->addAuditBulk(CAudit::ACTION_UPDATE, $resource, $graphs, $db_graphs);
+
 		return ['graphids' => $graphids];
 	}
 
@@ -166,6 +169,9 @@ abstract class CGraphGeneral extends CApiService {
 		$this->createReal($graphs);
 		$this->inherit($graphs);
 
+		$resource = ($this instanceof CGraph) ? CAudit::RESOURCE_GRAPH : CAudit::RESOURCE_GRAPH_PROTOTYPE;
+		$this->addAuditBulk(CAudit::ACTION_ADD, $resource, $graphs);
+
 		return ['graphids' => array_column($graphs, 'graphid')];
 	}
 
@@ -364,28 +370,9 @@ abstract class CGraphGeneral extends CApiService {
 		}
 
 		// adding HostGroups
-		if ($options['selectGroups'] !== null && $options['selectGroups'] !== API_OUTPUT_COUNT) {
-			$relationMap = new CRelationMap();
-			// discovered items
-			$dbRules = DBselect(
-				'SELECT gi.graphid,hg.groupid'.
-				' FROM graphs_items gi,items i,hosts_groups hg'.
-				' WHERE '.dbConditionInt('gi.graphid', $graphids).
-				' AND gi.itemid=i.itemid'.
-				' AND i.hostid=hg.hostid'
-			);
-			while ($relation = DBfetch($dbRules)) {
-				$relationMap->addRelation($relation['graphid'], $relation['groupid']);
-			}
-
-			$groups = API::HostGroup()->get([
-				'output' => $options['selectGroups'],
-				'groupids' => $relationMap->getRelatedIds(),
-				'nopermissions' => true,
-				'preservekeys' => true
-			]);
-			$result = $relationMap->mapMany($result, $groups, 'groups');
-		}
+		$this->addRelatedGroups($options, $result, 'selectGroups');
+		$this->addRelatedGroups($options, $result, 'selectHostGroups');
+		$this->addRelatedGroups($options, $result, 'selectTemplateGroups');
 
 		// adding Hosts
 		if ($options['selectHosts'] !== null && $options['selectHosts'] !== API_OUTPUT_COUNT) {
@@ -449,6 +436,65 @@ abstract class CGraphGeneral extends CApiService {
 		return $result;
 	}
 
+	/**
+	 * Adds related host groups and template groups requested by "select*" options to the resulting object set.
+	 *
+	 * @param array  $options  [IN] Original input options.
+	 * @param array  $result   [IN/OUT] Result output.
+	 * @param string $option   [IN] Possible values:
+	 *                                - "selectGroups" (deprecated);
+	 *                                - "selectHostGroups";
+	 *                                - "selectTemplateGroups".
+	 */
+	private function addRelatedGroups(array $options, array &$result, string $option): void {
+		if ($options[$option] === null || $options[$option] === API_OUTPUT_COUNT) {
+			return;
+		}
+
+		$relationMap = new CRelationMap();
+
+		// discovered items
+		$dbRules = DBselect(
+			'SELECT gi.graphid,hg.groupid'.
+			' FROM graphs_items gi,items i,hosts_groups hg'.
+			' WHERE '.dbConditionInt('gi.graphid', array_keys($result)).
+				' AND gi.itemid=i.itemid'.
+				' AND i.hostid=hg.hostid'
+		);
+		while ($relation = DBfetch($dbRules)) {
+			$relationMap->addRelation($relation['graphid'], $relation['groupid']);
+		}
+
+		switch ($option) {
+			case 'selectGroups':
+				$output_tag = 'groups';
+				$entities = [API::HostGroup(), API::TemplateGroup()];
+				break;
+
+			case 'selectHostGroups':
+				$entities = [API::HostGroup()];
+				$output_tag = 'hostgroups';
+				break;
+
+			case 'selectTemplateGroups':
+				$entities = [API::TemplateGroup()];
+				$output_tag = 'templategroups';
+				break;
+		}
+
+		$groups = [];
+		foreach ($entities as $entity) {
+			$groups += $entity->get([
+				'output' => $options[$option],
+				'groupids' => $relationMap->getRelatedIds(),
+				'nopermissions' => true,
+				'preservekeys' => true
+			]);
+		}
+
+		$result = $relationMap->mapMany($result, $groups, $output_tag);
+	}
+
 	/**
 	 * Validate graph name and graph items including Y axis item ID's and graph item fields on Create method
 	 * and return valid item ID's on success or trow an error on failure.
@@ -1273,7 +1319,8 @@ abstract class CGraphGeneral extends CApiService {
 			'output' => $output,
 			'selectGraphItems' => ['itemid', 'drawtype', 'sortorder', 'color', 'yaxisside', 'calc_fnc', 'type'],
 			'hostids' => $data['templateids'],
-			'preservekeys' => true
+			'preservekeys' => true,
+			'nopermissions' => true
 		]);
 
 		if ($graphs) {
