@@ -61,7 +61,8 @@ class CHost extends CHostGeneral {
 	 * @param bool          $options['tags']                               Select hosts by given tags.
 	 * @param bool          $options['severities']                         Select hosts that have only problems with given severities.
 	 * @param bool          $options['inheritedTags']                      Select hosts that have given tags also in their linked templates.
-	 * @param string|array  $options['selectGroups']                       Return a "groups" property with host groups data that the host belongs to.
+	 * @param string|array  $options['selectGroups']                       Deprecated! Return a "groups" property with host groups data that the host belongs to.
+	 * @param string|array  $options['selectHostGroups']                   Return a "hostgroups" property with host groups data that the host belongs to.
 	 * @param string|array  $options['selectParentTemplates']              Return a "parentTemplates" property with templates that the host is linked to.
 	 * @param string|array  $options['selectItems']                        Return an "items" property with host items.
 	 * @param string|array  $options['selectDiscoveries']                  Return a "discoveries" property with host low-level discovery rules.
@@ -142,6 +143,7 @@ class CHost extends CHostGeneral {
 			// output
 			'output'							=> API_OUTPUT_EXTEND,
 			'selectGroups'						=> null,
+			'selectHostGroups'					=> null,
 			'selectParentTemplates'				=> null,
 			'selectItems'						=> null,
 			'selectDiscoveries'					=> null,
@@ -169,6 +171,8 @@ class CHost extends CHostGeneral {
 
 		$this->validateGet($options);
 
+		$this->checkDeprecatedParam($options, 'selectGroups');
+
 		// editable + PERMISSION CHECK
 		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
 			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
@@ -464,6 +468,15 @@ class CHost extends CHostGeneral {
 				$sqlParts['left_join']['interface'] = ['alias' => 'hi', 'table' => 'interface', 'using' => 'hostid'];
 				$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
 			}
+
+			if (array_key_exists('active_available', $options['filter'])
+					&& $options['filter']['active_available'] !== null) {
+				$this->dbFilter('host_rtdata hr', ['filter' => [
+						'active_available' => $options['filter']['active_available']
+					]] + $options,
+					$sqlParts
+				);
+			}
 		}
 
 		// tags
@@ -495,6 +508,7 @@ class CHost extends CHostGeneral {
 		if ($options['output'] === API_OUTPUT_EXTEND) {
 			$all_keys = array_keys(DB::getSchema($this->tableName())['fields']);
 			$all_keys[] = 'inventory_mode';
+			$all_keys[] = 'active_available';
 			$options['output'] = array_diff($all_keys, $write_only_keys);
 		}
 		/*
@@ -581,17 +595,29 @@ class CHost extends CHostGeneral {
 	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
 		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
 
-		if (!$options['countOutput'] && $this->outputIsRequested('inventory_mode', $options['output'])) {
-			$sqlParts['select']['inventory_mode'] =
-				dbConditionCoalesce('hinv.inventory_mode', HOST_INVENTORY_DISABLED, 'inventory_mode');
-		}
-
 		if ((!$options['countOutput'] && $this->outputIsRequested('inventory_mode', $options['output']))
 				|| ($options['filter'] && array_key_exists('inventory_mode', $options['filter']))) {
 			$sqlParts['left_join'][] = ['alias' => 'hinv', 'table' => 'host_inventory', 'using' => 'hostid'];
 			$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
 		}
 
+		if ((!$options['countOutput'] && $this->outputIsRequested('active_available', $options['output']))
+				|| (is_array($options['filter']) && array_key_exists('active_available', $options['filter']))) {
+			$sqlParts['left_join'][] = ['alias' => 'hr', 'table' => 'host_rtdata', 'using' => 'hostid'];
+			$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
+		}
+
+		if (!$options['countOutput']) {
+			if ($this->outputIsRequested('inventory_mode', $options['output'])) {
+				$sqlParts['select']['inventory_mode'] =
+					dbConditionCoalesce('hinv.inventory_mode', HOST_INVENTORY_DISABLED, 'inventory_mode');
+			}
+
+			if ($this->outputIsRequested('active_available', $options['output'])) {
+				$sqlParts = $this->addQuerySelect('hr.active_available', $sqlParts);
+			}
+		}
+
 		return $sqlParts;
 	}
 
@@ -644,10 +670,13 @@ class CHost extends CHostGeneral {
 		$hosts_inventory = [];
 		$templates_hostids = [];
 
+		$hosts_rtdata = [];
+
 		$hostids = DB::insert('hosts', $hosts);
 
 		foreach ($hosts as $index => &$host) {
 			$host['hostid'] = $hostids[$index];
+			$hosts_rtdata[$index] = ['hostid' => $hostids[$index]];
 
 			foreach ($host['groups'] as $group) {
 				$hosts_groups[] = [
@@ -722,7 +751,9 @@ class CHost extends CHostGeneral {
 			DB::insert('host_inventory', $hosts_inventory, false);
 		}
 
-		$this->addAuditBulk(AUDIT_ACTION_ADD, AUDIT_RESOURCE_HOST, $hosts);
+		DB::insertBatch('host_rtdata', $hosts_rtdata, false);
+
+		$this->addAuditBulk(CAudit::ACTION_ADD, CAudit::RESOURCE_HOST, $hosts);
 
 		return ['hostids' => array_column($hosts, 'hostid')];
 	}
@@ -774,7 +805,7 @@ class CHost extends CHostGeneral {
 		$inventories = [];
 		foreach ($hosts as &$host) {
 			// If visible name is not given or empty it should be set to host name.
-			if (array_key_exists('host', $host) && (!array_key_exists('name', $host) || !trim($host['name']))) {
+			if (array_key_exists('host', $host) && (!array_key_exists('name', $host) || trim($host['name']) === '')) {
 				$host['name'] = $host['host'];
 			}
 
@@ -822,7 +853,7 @@ class CHost extends CHostGeneral {
 			}
 		}
 
-		$this->updateTags(array_column($hosts, 'tags', 'hostid'));
+		$this->updateTags($hosts, $db_hosts);
 
 		return ['hostids' => array_column($hosts, 'hostid')];
 	}
@@ -1219,7 +1250,7 @@ class CHost extends CHostGeneral {
 
 		/*
 		 * Update host and host group linkage. This procedure should be done the last because user can unlink
-		 * him self from a group with write permissions leaving only read premissions. Thus other procedures, like
+		 * him self from a group with write permissions leaving only read permissions. Thus other procedures, like
 		 * host-template linkage, inventory update, macros update, must be done before this.
 		 */
 		if (isset($updateGroups)) {
@@ -1259,22 +1290,20 @@ class CHost extends CHostGeneral {
 			$new_hosts[] = $new_host;
 		}
 
-		$this->addAuditBulk(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_HOST, $new_hosts, $db_hosts);
+		$this->addAuditBulk(CAudit::ACTION_UPDATE, CAudit::RESOURCE_HOST, $new_hosts, $db_hosts);
 
 		return ['hostids' => $inputHostIds];
 	}
 
 	/**
-	 * Additionally allows to remove interfaces from hosts.
-	 *
-	 * Checks write permissions for hosts.
-	 *
-	 * Additional supported $data parameters are:
-	 * - interfaces  - an array of interfaces to delete from the hosts
-	 *
-	 * @throws APIException if the input is invalid.
+	 * Removes templates and interfaces from hosts.
 	 *
 	 * @param array $data
+	 * @param array $data['interfaces']         Interfaces to delete from the hosts.
+	 * @param array $data['templateids']        Templates to unlink from host.
+	 * @param array $data['templateids_clear']  Templates to unlink and clear from host.
+	 *
+	 * @throws APIException if the input is invalid.
 	 *
 	 * @return array
 	 */
@@ -1303,78 +1332,145 @@ class CHost extends CHostGeneral {
 
 		$data['templateids'] = [];
 
+		if (array_key_exists('templateids_link', $data) && $data['templateids_link']
+				|| array_key_exists('templateids_clear', $data) && $data['templateids_clear']) {
+			// If unlink or clear is requested, get existing host templates to determine the link type.
+			$hosts_templates = $this->get([
+				'selectParentTemplates' => ['templateid', 'link_type'],
+				'hostids' => $data['hostids'],
+				'preservekeys' => true,
+				'nopermissions' => true
+			]);
+
+			$prohibited_templateids = [];
+
+			foreach ($hosts_templates as $host_templates) {
+				if ($host_templates['parentTemplates']) {
+					foreach ($host_templates['parentTemplates'] as $template) {
+						if ($template['link_type'] == TEMPLATE_LINK_LLD) {
+							$prohibited_templateids[$template['templateid']] = true;
+						}
+					}
+				}
+			}
+
+			// Some templates may not be allowed to unlink. Remove IDs from both lists.
+			if ($prohibited_templateids) {
+				foreach (['templateids_link', 'templateids_clear'] as $field) {
+					if (array_key_exists($field, $data) && $data[$field]) {
+						foreach ($data[$field] as $idx => $templateid) {
+							if (array_key_exists($templateid, $prohibited_templateids)) {
+								unset($data[$field][$idx]);
+							}
+						}
+					}
+				}
+			}
+		}
+
 		return parent::massRemove($data);
 	}
 
 	/**
 	 * Validates the input parameters for the delete() method.
 	 *
-	 * @throws APIException if the input is invalid
+	 * @param array      $hostids
+	 * @param array|null $db_hosts
 	 *
-	 * @param array $hostIds
-	 * @param bool 	$nopermissions
+	 * @throws APIException if the input is invalid.
 	 */
-	protected function validateDelete(array $hostIds, $nopermissions = false) {
-		if (!$hostIds) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Empty input parameter.'));
+	private function validateDelete(array &$hostids, array &$db_hosts = null): void {
+		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
+
+		if (!CApiInputValidator::validate($api_input_rules, $hostids, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
-		if (!$nopermissions) {
-			$this->checkPermissions($hostIds, _('No permissions to referred object or it does not exist!'));
+		$db_hosts = $this->get([
+			'output' => ['hostid', 'host'],
+			'hostids' => $hostids,
+			'editable' => true,
+			'preservekeys' => true
+		]);
+
+		if (count($db_hosts) != count($hostids)) {
+			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
 		}
 
-		$this->validateDeleteCheckMaintenances($hostIds);
+		self::validateDeleteForce($db_hosts);
 	}
 
 	/**
-	 * Validates if hosts may be deleted, due to maintenance constrain.
-	 *
-	 * @throws APIException if a constrain failed
+	 * @param array $db_hosts
+	 */
+	public static function validateDeleteForce(array $db_hosts): void {
+		self::checkMaintenances(array_keys($db_hosts));
+	}
+
+	/**
+	 * Check that no maintenance object will be left without hosts and host groups as the result of the given hosts
+	 * deletion.
 	 *
 	 * @param array $hostids
+	 *
+	 * @throws APIException
 	 */
-	protected function validateDeleteCheckMaintenances(array $hostids) {
+	private static function checkMaintenances(array $hostids): void {
 		$maintenance = DBfetch(DBselect(
-			'SELECT m.name'.
+			'SELECT m.maintenanceid,m.name'.
 			' FROM maintenances m'.
 			' WHERE NOT EXISTS ('.
 				'SELECT NULL'.
 				' FROM maintenances_hosts mh'.
 				' WHERE m.maintenanceid=mh.maintenanceid'.
-					' AND '.dbConditionInt('mh.hostid', $hostids, true).
+					' AND '.dbConditionId('mh.hostid', $hostids, true).
 			')'.
 				' AND NOT EXISTS ('.
 					'SELECT NULL'.
 					' FROM maintenances_groups mg'.
 					' WHERE m.maintenanceid=mg.maintenanceid'.
 				')'
-		));
+		, 1));
 
 		if ($maintenance) {
+			$maintenance_hosts = DBfetchColumn(DBselect(
+				'SELECT h.host'.
+				' FROM maintenances_hosts mh,hosts h'.
+				' WHERE mh.hostid=h.hostid'.
+					' AND '.dbConditionId('mh.maintenanceid', [$maintenance['maintenanceid']])
+			), 'host');
+
 			self::exception(ZBX_API_ERROR_PARAMETERS, _n(
-				'Cannot delete host because maintenance "%1$s" must contain at least one host or host group.',
-				'Cannot delete selected hosts because maintenance "%1$s" must contain at least one host or host group.',
-				$maintenance['name'],
-				count($hostids)
+				'Cannot delete host %1$s because maintenance "%2$s" must contain at least one host or host group.',
+				'Cannot delete hosts %1$s because maintenance "%2$s" must contain at least one host or host group.',
+				'"'.implode('", "', $maintenance_hosts).'"', $maintenance['name'], count($maintenance_hosts)
 			));
 		}
 	}
 
 	/**
-	 * Delete Host.
-	 *
-	 * @param array	$hostIds
-	 * @param bool	$nopermissions
+	 * @param array $hostids
 	 *
 	 * @return array
 	 */
-	public function delete(array $hostIds, $nopermissions = false) {
-		$this->validateDelete($hostIds, $nopermissions);
+	public function delete(array $hostids): array {
+		$this->validateDelete($hostids, $db_hosts);
+
+		self::deleteForce($db_hosts);
+
+		return ['hostids' => $hostids];
+	}
+
+	/**
+	 * @param array $db_hosts
+	 */
+	public static function deleteForce(array $db_hosts): void {
+		$hostids = array_keys($db_hosts);
 
 		// delete the discovery rules first
 		$del_rules = API::DiscoveryRule()->get([
 			'output' => [],
-			'hostids' => $hostIds,
+			'hostids' => $hostids,
 			'nopermissions' => true,
 			'preservekeys' => true
 		]);
@@ -1385,7 +1481,7 @@ class CHost extends CHostGeneral {
 		// delete the items
 		$del_items = API::Item()->get([
 			'output' => [],
-			'templateids' => $hostIds,
+			'templateids' => $hostids,
 			'nopermissions' => true,
 			'preservekeys' => true
 		]);
@@ -1395,7 +1491,7 @@ class CHost extends CHostGeneral {
 
 		// delete web tests
 		$delHttptests = [];
-		$dbHttptests = get_httptests_by_hostid($hostIds);
+		$dbHttptests = get_httptests_by_hostid($hostids);
 		while ($dbHttptest = DBfetch($dbHttptests)) {
 			$delHttptests[$dbHttptest['httptestid']] = $dbHttptest['httptestid'];
 		}
@@ -1404,10 +1500,10 @@ class CHost extends CHostGeneral {
 		}
 
 		// delete host from maps
-		if (!empty($hostIds)) {
+		if (!empty($hostids)) {
 			DB::delete('sysmaps_elements', [
 				'elementtype' => SYSMAP_ELEMENT_TYPE_HOST,
-				'elementid' => $hostIds
+				'elementid' => $hostids
 			]);
 		}
 
@@ -1417,7 +1513,7 @@ class CHost extends CHostGeneral {
 		$sql = 'SELECT DISTINCT actionid'.
 				' FROM conditions'.
 				' WHERE conditiontype='.CONDITION_TYPE_HOST.
-				' AND '.dbConditionString('value', $hostIds);
+				' AND '.dbConditionString('value', $hostids);
 		$dbActions = DBselect($sql);
 		while ($dbAction = DBfetch($dbActions)) {
 			$actionids[$dbAction['actionid']] = $dbAction['actionid'];
@@ -1427,7 +1523,7 @@ class CHost extends CHostGeneral {
 		$sql = 'SELECT DISTINCT o.actionid'.
 				' FROM operations o, opcommand_hst oh'.
 				' WHERE o.operationid=oh.operationid'.
-				' AND '.dbConditionInt('oh.hostid', $hostIds);
+				' AND '.dbConditionInt('oh.hostid', $hostids);
 		$dbActions = DBselect($sql);
 		while ($dbAction = DBfetch($dbActions)) {
 			$actionids[$dbAction['actionid']] = $dbAction['actionid'];
@@ -1445,21 +1541,21 @@ class CHost extends CHostGeneral {
 		// delete action conditions
 		DB::delete('conditions', [
 			'conditiontype' => CONDITION_TYPE_HOST,
-			'value' => $hostIds
+			'value' => $hostids
 		]);
 
 		// delete action operation commands
 		$operationids = [];
 		$sql = 'SELECT DISTINCT oh.operationid'.
 				' FROM opcommand_hst oh'.
-				' WHERE '.dbConditionInt('oh.hostid', $hostIds);
+				' WHERE '.dbConditionInt('oh.hostid', $hostids);
 		$dbOperations = DBselect($sql);
 		while ($dbOperation = DBfetch($dbOperations)) {
 			$operationids[$dbOperation['operationid']] = $dbOperation['operationid'];
 		}
 
 		DB::delete('opcommand_hst', [
-			'hostid' => $hostIds
+			'hostid' => $hostids
 		]);
 
 		// delete empty operations
@@ -1477,26 +1573,18 @@ class CHost extends CHostGeneral {
 			'operationid' => $delOperationids
 		]);
 
-		$db_hosts = API::Host()->get([
-			'output' => ['hostid', 'name'],
-			'hostids' => $hostIds,
-			'nopermissions' => true
-		]);
-
 		// delete host inventory
-		DB::delete('host_inventory', ['hostid' => $hostIds]);
+		DB::delete('host_inventory', ['hostid' => $hostids]);
 
 		// delete host
-		DB::delete('hosts', ['hostid' => $hostIds]);
+		DB::delete('host_tag', ['hostid' => $hostids]);
+		DB::update('hosts', [
+			'values' => ['templateid' => 0],
+			'where' => ['hostid' => $hostids, 'flags' => ZBX_FLAG_DISCOVERY_PROTOTYPE]
+		]);
+		DB::delete('hosts', ['hostid' => $hostids]);
 
-		// TODO: remove info from API
-		foreach ($db_hosts as $db_host) {
-			info(_s('Deleted: Host "%1$s".', $db_host['name']));
-		}
-
-		$this->addAuditBulk(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_HOST, $db_hosts);
-
-		return ['hostids' => $hostIds];
+		self::addAuditLog(CAudit::ACTION_DELETE, CAudit::RESOURCE_HOST, $db_hosts);
 	}
 
 	/**
@@ -1510,9 +1598,12 @@ class CHost extends CHostGeneral {
 	protected function addRelatedObjects(array $options, array $result) {
 		$result = parent::addRelatedObjects($options, $result);
 
+		// adding groups
+		$this->addRelatedGroups($options, $result, 'selectGroups');
+		$this->addRelatedGroups($options, $result, 'selectHostGroups');
+
 		$hostids = array_keys($result);
 
-		// adding inventory
 		if ($options['selectInventory'] !== null) {
 			$inventory = API::getApiService()->select('host_inventory', [
 				'output' => $options['selectInventory'],
@@ -1525,7 +1616,6 @@ class CHost extends CHostGeneral {
 			$result = $relation_map->mapOne($result, $inventory, 'inventory');
 		}
 
-		// adding hostinterfaces
 		if ($options['selectInterfaces'] !== null) {
 			if ($options['selectInterfaces'] != API_OUTPUT_COUNT) {
 				$interfaces = API::HostInterface()->get([
@@ -1562,7 +1652,6 @@ class CHost extends CHostGeneral {
 			}
 		}
 
-		// Adding dashboards.
 		if ($options['selectDashboards'] !== null) {
 			[$hosts_templates, $templateids] = CApiHostHelper::getParentTemplates($hostids);
 
@@ -1608,7 +1697,6 @@ class CHost extends CHostGeneral {
 			}
 		}
 
-		// adding discovery rule
 		if ($options['selectDiscoveryRule'] !== null && $options['selectDiscoveryRule'] != API_OUTPUT_COUNT) {
 			// discovered items
 			$discoveryRules = DBFetchArray(DBselect(
@@ -1627,7 +1715,6 @@ class CHost extends CHostGeneral {
 			$result = $relationMap->mapOne($result, $discoveryRules, 'discoveryRule');
 		}
 
-		// adding host discovery
 		if ($options['selectHostDiscovery'] !== null) {
 			$hostDiscoveries = API::getApiService()->select('host_discovery', [
 				'output' => $this->outputExtend($options['selectHostDiscovery'], ['hostid']),
@@ -1642,8 +1729,35 @@ class CHost extends CHostGeneral {
 			$result = $relationMap->mapOne($result, $hostDiscoveries, 'hostDiscovery');
 		}
 
+		if ($options['selectTags'] !== null) {
+			foreach ($result as &$row) {
+				$row['tags'] = [];
+			}
+			unset($row);
+
+			if ($options['selectTags'] === API_OUTPUT_EXTEND) {
+				$output = ['hosttagid', 'hostid', 'tag', 'value', 'automatic'];
+			}
+			else {
+				$output = array_unique(array_merge(['hosttagid', 'hostid'], $options['selectTags']));
+			}
+
+			$sql_options = [
+				'output' => $output,
+				'filter' => ['hostid' => $hostids]
+			];
+			$db_tags = DBselect(DB::makeSql('host_tag', $sql_options));
+
+			while ($db_tag = DBfetch($db_tags)) {
+				$hostid = $db_tag['hostid'];
+
+				unset($db_tag['hosttagid'], $db_tag['hostid']);
+
+				$result[$hostid]['tags'][] = $db_tag;
+			}
+		}
+
 		if ($options['selectInheritedTags'] !== null && $options['selectInheritedTags'] != API_OUTPUT_COUNT) {
-			$hosts_templates = [];
 			[$hosts_templates, $templateids] = CApiHostHelper::getParentTemplates($hostids);
 
 			$templates = API::Template()->get([
@@ -1680,6 +1794,31 @@ class CHost extends CHostGeneral {
 		return $result;
 	}
 
+	/**
+	 * Adds related host groups requested by "select*" options to the resulting object set.
+	 *
+	 * @param array  $options  [IN] Original input options.
+	 * @param array  $result   [IN/OUT] Result output.
+	 * @param string $option   [IN] Possible values:
+	 *                                - "selectGroups" (deprecated);
+	 *                                - "selectHostGroups" (or any other value).
+	 */
+	private function addRelatedGroups(array $options, array &$result, string $option): void {
+		if ($options[$option] === null || $options[$option] === API_OUTPUT_COUNT) {
+			return;
+		}
+
+		$relationMap = $this->createRelationMap($result, 'hostid', 'groupid', 'hosts_groups');
+		$groups = API::HostGroup()->get([
+			'output' => $options[$option],
+			'groupids' => $relationMap->getRelatedIds(),
+			'preservekeys' => true
+		]);
+
+		$output_tag = $option === 'selectGroups' ? 'groups' : 'hostgroups';
+		$result = $relationMap->mapMany($result, $groups, $output_tag);
+	}
+
 	/**
 	 * Validates the input parameters for the get() method.
 	 *
@@ -1690,15 +1829,18 @@ class CHost extends CHostGeneral {
 	protected function validateGet(array $options) {
 		// Validate input parameters.
 		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
-			'inheritedTags' => ['type' => API_BOOLEAN, 'default' => false],
-			'selectInheritedTags' => ['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-			'severities' =>	[
-				'type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE | API_NOT_EMPTY, 'in' => implode(',', range(TRIGGER_SEVERITY_NOT_CLASSIFIED, TRIGGER_SEVERITY_COUNT - 1)), 'uniq' => true
-			],
+			'inheritedTags' =>				['type' => API_BOOLEAN, 'default' => false],
+			'selectInheritedTags' =>		['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+			'severities' =>					['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE | API_NOT_EMPTY, 'in' => implode(',', range(TRIGGER_SEVERITY_NOT_CLASSIFIED, TRIGGER_SEVERITY_COUNT - 1)), 'uniq' => true],
 			'withProblemsSuppressed' =>		['type' => API_BOOLEAN, 'flags' => API_ALLOW_NULL],
-			'selectValueMaps' =>			['type' => API_OUTPUT, 'flags' => API_ALLOW_NULL, 'in' => 'valuemapid,name,mappings']
+			'selectTags' =>					['type' => API_OUTPUT, 'flags' => API_ALLOW_NULL, 'in' => implode(',', ['tag', 'value', 'automatic'])],
+			'selectValueMaps' =>			['type' => API_OUTPUT, 'flags' => API_ALLOW_NULL, 'in' => implode(',', ['valuemapid', 'name', 'mappings'])],
+			'selectParentTemplates' =>		['type' => API_OUTPUT, 'flags' => API_ALLOW_NULL | API_ALLOW_COUNT, 'in' => implode(',', ['templateid', 'host', 'name', 'description', 'uuid', 'link_type'])],
+			'selectMacros' =>				['type' => API_OUTPUT, 'flags' => API_ALLOW_NULL, 'in' => implode(',', ['hostmacroid', 'macro', 'value', 'type', 'description', 'automatic'])]
 		]];
+
 		$options_filter = array_intersect_key($options, $api_input_rules['fields']);
+
 		if (!CApiInputValidator::validate($api_input_rules, $options_filter, '/', $error)) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
@@ -1855,7 +1997,7 @@ class CHost extends CHostGeneral {
 			'type' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET, ZBX_MACRO_TYPE_VAULT]), 'default' => ZBX_MACRO_TYPE_TEXT],
 			'value' =>			['type' => API_MULTIPLE, 'flags' => API_REQUIRED, 'rules' => [
 									['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET])], 'type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'value')],
-									['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_VAULT])], 'type' => API_VAULT_SECRET, 'length' => DB::getFieldLength('hostmacro', 'value')]
+									['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_VAULT])], 'type' => API_VAULT_SECRET, 'provider' => CSettingsHelper::get(CSettingsHelper::VAULT_PROVIDER), 'length' => DB::getFieldLength('hostmacro', 'value')]
 			]],
 			'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'description')]
 		]];
@@ -1912,11 +2054,6 @@ class CHost extends CHostGeneral {
 				$groupids[$group['groupid']] = true;
 			}
 
-			// Validate tags.
-			if (array_key_exists('tags', $host)) {
-				$this->validateTags($host);
-			}
-
 			if (array_key_exists('macros', $host)) {
 				if (!CApiInputValidator::validate($macro_rules, $host['macros'], '/'.($index + 1).'/macros', $error)) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, $error);
@@ -1925,6 +2062,8 @@ class CHost extends CHostGeneral {
 		}
 		unset($host);
 
+		self::checkTags($hosts);
+
 		// Check for duplicate "host" and "name" fields.
 		$duplicate = CArrayHelper::findDuplicate($hosts, 'host');
 		if ($duplicate) {
@@ -2080,7 +2219,8 @@ class CHost extends CHostGeneral {
 			'macro' =>			['type' => API_USER_MACRO, 'length' => DB::getFieldLength('hostmacro', 'macro')],
 			'type' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET, ZBX_MACRO_TYPE_VAULT])],
 			'value' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'value')],
-			'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'description')]
+			'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'description')],
+			'automatic' =>		['type' => API_INT32, 'in' => implode(',', [ZBX_USERMACRO_MANUAL])]
 		]];
 
 		$db_hosts = $this->get([
@@ -2143,8 +2283,8 @@ class CHost extends CHostGeneral {
 					}
 				}
 			}
-			// Permissions to host groups is validated in massUpdate().
 
+			// Permissions to host groups is validated in massUpdate().
 			if (array_key_exists('macros', $host)) {
 				if (!CApiInputValidator::validate($macro_rules, $host['macros'], '/'.($index + 1).'/macros', $error)) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, $error);
@@ -2173,7 +2313,9 @@ class CHost extends CHostGeneral {
 		]);
 
 		$update_discovered_validator = new CUpdateDiscoveredValidator([
-			'allowed' => ['hostid', 'status', 'inventory', 'description'],
+			'allowed' => ['hostid', 'status', 'description', 'tags', 'macros', 'inventory', 'templates',
+				'templates_clear'
+			],
 			'messageAllowedField' => _('Cannot update "%2$s" for a discovered host "%1$s".')
 		]);
 
@@ -2276,14 +2418,12 @@ class CHost extends CHostGeneral {
 					}
 				}
 			}
-
-			// Validate tags.
-			if (array_key_exists('tags', $host)) {
-				$this->validateTags($host);
-			}
 		}
 		unset($host);
 
+		$this->addAffectedTags($hosts, $db_hosts);
+		self::checkTags($hosts);
+
 		if (array_key_exists('host', $host_names) || array_key_exists('name', $host_names)) {
 			$filter = [];
 
@@ -2351,6 +2491,24 @@ class CHost extends CHostGeneral {
 		return $hosts;
 	}
 
+	/**
+	 * @param array $hosts
+	 *
+	 * @throws APIException
+	 */
+	private static function checkTags(array &$hosts): void {
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_ALLOW_UNEXPECTED, 'fields' => [
+			'tags' =>	['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['tag', 'value']], 'fields' => [
+				'tag' =>	['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('host_tag', 'tag')],
+				'value' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('host_tag', 'value'), 'default' => DB::getDefault('host_tag', 'value')]
+			]]
+		]];
+
+		if (!CApiInputValidator::validate($api_input_rules, $hosts, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+	}
+
 	protected function requiresPostSqlFiltering(array $options) {
 		return ($options['severities'] !== null || $options['withProblemsSuppressed'] !== null);
 	}
