@@ -1388,7 +1388,7 @@ function make_sorting_header($obj, $tabfield, $sortField, $sortOrder, $link = nu
  *
  * @param string   $number     Valid number in decimal or scientific notation.
  * @param int|null $precision  Max number of significant digits to take into account. Default: ZBX_FLOAT_DIG.
- * @param int|null $decimals   Max number of first non-zero decimals decimals to display. Default: 0.
+ * @param int|null $decimals   Max number of first non-zero decimals to display. Default: 0.
  * @param bool     $exact      Display exactly this number of decimals instead of first non-zeros.
  *
  * Note: $decimals must be less than $precision.
@@ -1491,6 +1491,10 @@ function formatFloat(float $number, int $precision = null, int $decimals = null,
 * @return float
 */
 function truncateFloat(float $number): float {
+	if ($number == INF) {
+		return INF;
+	}
+
 	return (float) sprintf('%.'.(ZBX_FLOAT_DIG - 1).'E', $number);
 }
 
@@ -1588,11 +1592,13 @@ function access_deny($mode = ACCESS_DENY_OBJECT) {
 			// display the login button only for guest users
 			if (CWebUser::isGuest()) {
 				$data['buttons'][] = (new CButton('login', _('Login')))
-					->onClick('javascript: document.location = "index.php?request='.$url.'";');
+					->setAttribute('data-url', $url)
+					->onClick('document.location = "index.php?request=" + this.dataset.url;');
 			}
 
 			$data['buttons'][] = (new CButton('back', _s('Go to "%1$s"', CMenuHelper::getFirstLabel())))
-				->onClick('javascript: document.location = "'.CMenuHelper::getFirstUrl().'"');
+				->setAttribute('data-url', CMenuHelper::getFirstUrl())
+				->onClick('document.location = this.dataset.url');
 		}
 		// if the user is not logged in - offer to login
 		else {
@@ -1603,7 +1609,9 @@ function access_deny($mode = ACCESS_DENY_OBJECT) {
 					_('If you think this message is wrong, please consult your administrators about getting the necessary permissions.')
 				],
 				'buttons' => [
-					(new CButton('login', _('Login')))->onClick('javascript: document.location = "index.php?request='.$url.'";')
+					(new CButton('login', _('Login')))
+						->setAttribute('data-url', $url)
+						->onClick('document.location = "index.php?request=" + this.dataset.url;')
 				]
 			];
 		}
@@ -1690,9 +1698,7 @@ function makeMessageBox(string $class, array $messages, string $title = null, bo
 		}
 
 		foreach ($messages as $message) {
-			foreach (explode("\n", $message['message']) as $message_part) {
-				$list->addItem($message_part);
-			}
+			$list->addItem($message['message']);
 		}
 
 		$msg_details = (new CDiv())
@@ -2163,11 +2169,16 @@ function hasErrorMessages() {
  * @param array  $keepids   checked rows ids
  */
 function uncheckTableRows($parentid = null, $keepids = []) {
-	$key = implode('_', array_filter(['cb', basename($_SERVER['SCRIPT_NAME'], '.php'), $parentid]));
+	if ($parentid === null) {
+		$key = implode('_', ['cb', basename($_SERVER['SCRIPT_NAME'], '.php')]);
+	}
+	else {
+		// Allow $parentid to be zero. For example actionconf.php uses $parentid as event source.
+		$key = implode('_', ['cb', basename($_SERVER['SCRIPT_NAME'], '.php'), $parentid]);
+	}
 
 	if ($keepids) {
-		// If $keepids will not have same key as value, it will create mess, when new checkbox will be checked.
-		$keepids = array_combine($keepids, $keepids);
+		$keepids = array_fill_keys($keepids, '');
 
 		insert_js('sessionStorage.setItem('.json_encode($key).', JSON.stringify('.json_encode($keepids).'));');
 	}
@@ -2231,19 +2242,14 @@ function splitPath($path) {
  * @param string   $color  a hexadecimal color identifier like "1F2C33"
  * @param int      $alpha
  *
- * @return int|false
+ * @return int
  */
 function get_color($image, $color, $alpha = 0) {
 	$red = hexdec('0x'.substr($color, 0, 2));
 	$green = hexdec('0x'.substr($color, 2, 2));
 	$blue = hexdec('0x'.substr($color, 4, 2));
 
-	if (function_exists('imagecolorexactalpha') && function_exists('imagecreatetruecolor')
-			&& @imagecreatetruecolor(1, 1)) {
-		return imagecolorexactalpha($image, $red, $green, $blue, $alpha);
-	}
-
-	return imagecolorallocate($image, $red, $green, $blue);
+	return imagecolorexactalpha($image, $red, $green, $blue, $alpha);
 }
 
 /**
