@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -79,25 +79,6 @@ function copy_expression(id, type) {
 	}
 }
 
-function cloneRow(elementid, count) {
-	if (typeof(cloneRow.count) === 'undefined') {
-		cloneRow.count = count;
-	}
-	cloneRow.count++;
-
-	var element = $('#' + elementid),
-		tpl = new Template($('<div>').append(element.clone(true)).html()),
-		entry = jQuery(tpl.evaluate({'id' : cloneRow.count}));
-
-	entry.find('*').each(function() {
-		jQuery(this).removeAttr('disabled');
-	});
-
-	entry.attr('id', 'entry_' + cloneRow.count);
-	entry[0].style.display = '';
-	entry.insertBefore(element.parent().children(':last-child'));
-}
-
 function testUserSound(idx) {
 	var element = document.getElementById(idx);
 	var sound = element.options[element.selectedIndex].value;
@@ -115,18 +96,11 @@ function testUserSound(idx) {
 	}
 }
 
-function removeObjectById(id) {
-	var obj = document.getElementById(id);
-	if (obj != null && typeof(obj) == 'object') {
-		obj.parentNode.removeChild(obj);
-	}
-}
-
 /**
  * Converts all HTML symbols into HTML entities.
  */
 jQuery.escapeHtml = function(html) {
-	return jQuery('<div />').text(html).html();
+	return jQuery('<div>').text(html).html();
 }
 
 function validateNumericBox(obj, allowempty, allownegative) {
@@ -289,6 +263,25 @@ function postMessageError(message) {
 	cookie.create('system-message-error', message);
 }
 
+function postMessageDetails(type, messages) {
+	const encode = function (string) {
+		const uint8 = new TextEncoder().encode(string);
+
+		let result = '';
+		for (let i = 0; i < uint8.byteLength; i++) {
+			result += String.fromCharCode(uint8[i]);
+		}
+
+		return result;
+	};
+
+	const data = JSON.stringify({
+		type: type,
+		messages: messages
+	});
+	cookie.create('system-message-details', btoa(encode(data)));
+}
+
 /**
  * Replace placeholders like %<number>$s with arguments.
  * Can be used like usual sprintf but only for %<number>$s placeholders.
@@ -498,7 +491,8 @@ function overlayDialogueDestroy(dialogueid) {
 		jQuery('[data-dialogueid='+dialogueid+']').remove();
 
 		removeFromOverlaysStack(dialogueid);
-		jQuery.publish('overlay.close', {dialogueid: dialogueid});
+
+		overlay.$dialogue[0].dispatchEvent(new CustomEvent('overlay.close', {detail: {dialogueid}}));
 	}
 }
 
@@ -507,6 +501,7 @@ function overlayDialogueDestroy(dialogueid) {
  *
  * @param {object} params                                   Modal window params.
  * @param {string} params.title                             Modal window title.
+ * @param {string} params.class                             Modal window CSS class, often based on .modal-popup*.
  * @param {object} params.content                           Window content.
  * @param {object} params.footer                           	Window footer content.
  * @param {object} params.controls                          Window controls.
@@ -523,7 +518,7 @@ function overlayDialogueDestroy(dialogueid) {
  * @param string   params.dialogueid            (optional)  Unique dialogue identifier to reuse existing overlay dialog
  *                                                          or create a new one if value is not set.
  * @param string   params.script_inline         (optional)  Custom javascript code to execute when initializing dialog.
- * @param {object} trigger_elmnt                (optional)  UI element which triggered opening of overlay dialogue.
+ * @param {Node|null} trigger_elmnt                         UI element which triggered opening of overlay dialogue.
  *
  * @return {Overlay}
  */
@@ -550,42 +545,53 @@ function overlayDialogue(params, trigger_elmnt) {
 /**
  * Execute script.
  *
- * @param string hostid				host id
- * @param string scriptid			script id
- * @param string confirmation		confirmation text
- * @param {object} trigger_elmnt	UI element that was clicked to open overlay dialogue.
+ * @param string scriptid			Script ID.
+ * @param string confirmation		Confirmation text.
+ * @param {Node} trigger_element	UI element that was clicked to open overlay dialogue.
+ * @param string hostid				Host ID.
+ * @param string eventid			Event ID.
  */
-function executeScript(hostid, scriptid, confirmation, trigger_elmnt) {
+function executeScript(scriptid, confirmation, trigger_element, hostid = null, eventid = null) {
 	var execute = function() {
+		var popup_options = {scriptid: scriptid};
+
 		if (hostid !== null) {
-			PopUp('popup.scriptexec', {
-				hostid: hostid,
-				scriptid: scriptid
-			}, null, trigger_elmnt);
+			popup_options.hostid = hostid;
+		}
+
+		if (eventid !== null) {
+			popup_options.eventid = eventid;
+		}
+
+		if (Object.keys(popup_options).length === 2) {
+			PopUp('popup.scriptexec', popup_options, {dialogue_class: 'modal-popup-medium', trigger_element});
 		}
 	};
 
 	if (confirmation.length > 0) {
 		overlayDialogue({
 			'title': t('Execution confirmation'),
-			'content': jQuery('<span>').text(confirmation),
+			'content': jQuery('<span>')
+				.addClass('confirmation-msg')
+				.text(confirmation),
+			'class': 'modal-popup modal-popup-small position-middle',
 			'buttons': [
 				{
 					'title': t('Cancel'),
 					'class': 'btn-alt',
-					'focused': (hostid === null),
+					'focused': (hostid === null && eventid === null),
 					'action': function() {}
 				},
 				{
 					'title': t('Execute'),
-					'enabled': (hostid !== null),
-					'focused': (hostid !== null),
+					'enabled': (hostid !== null || eventid !== null),
+					'focused': (hostid !== null || eventid !== null),
 					'action': function() {
 						execute();
 					}
 				}
 			]
-		}, trigger_elmnt);
+		}, trigger_element);
 
 		return false;
 	}
@@ -712,32 +718,23 @@ function parseUrlString(url) {
  * Message formatting function.
  *
  * @param {string}       type            Message type. ('good'|'bad'|'warning')
- * @param {string|array} messages        Array with details messages or message string with normal font.
- * @param {string}       title           Larger font title.
- * @param {bool}         show_close_box  Show close button.
- * @param {bool}         show_details    Show details on opening.
+ * @param {array}        messages        Error messages.
+ * @param {string|null}  title           Error title.
+ * @param {boolean}      show_close_box  Show close button.
+ * @param {boolean|null} show_details    Show details on opening.
  *
  * @return {jQuery}
  */
-function makeMessageBox(type, messages, title, show_close_box, show_details) {
+function makeMessageBox(type, messages, title = null, show_close_box = true, show_details = null) {
 	var classes = {good: 'msg-good', bad: 'msg-bad', warning: 'msg-warning'},
 		msg_class = classes[type];
 
-	if (typeof msg_class === 'undefined') {
-		return jQuery('<output>').text(Array.isArray(messages) ? messages.join(' ') : messages);
+	if (show_details === null) {
+		show_details = type === 'bad' || type === 'warning';
 	}
 
-	if (typeof title === 'undefined') {
-		title = null;
-	}
-	if (typeof show_close_box === 'undefined') {
-		show_close_box = true;
-	}
-	if (typeof show_details === 'undefined') {
-		show_details = false;
-	}
-
-	var	$list = jQuery('<ul>'),
+	var	$list = jQuery('<ul>')
+			.addClass('list-dashed'),
 		$msg_details = jQuery('<div>')
 			.addClass('msg-details')
 			.append($list),
@@ -784,7 +781,7 @@ function makeMessageBox(type, messages, title, show_close_box, show_details) {
 	}
 
 	if (Array.isArray(messages) && messages.length > 0) {
-		jQuery.map(messages, function(message) {
+		jQuery.map(messages, function (message) {
 			jQuery('<li>')
 				.text(message)
 				.appendTo($list);
@@ -793,13 +790,6 @@ function makeMessageBox(type, messages, title, show_close_box, show_details) {
 
 		$msg_box.append($msg_details);
 	}
-	else {
-		jQuery('<li>')
-			.text(messages ? messages : ' ')
-			.appendTo($list);
-
-		$msg_box.append($msg_details);
-	}
 
 	if (show_close_box) {
 		var $button = jQuery('<button>')
@@ -820,14 +810,15 @@ function makeMessageBox(type, messages, title, show_close_box, show_details) {
 /**
  * Download svg graph as .png image.
  *
- * @param {object} $dom_node    jQuery svg node to download.
- * @param {string} file_name    File name.
+ * @param {SVGElement} svg
+ * @param {string}     file_name
  */
-function downloadSvgImage($dom_node, file_name) {
-	var canvas = document.createElement('canvas'),
+function downloadSvgImage(svg, file_name) {
+	var $dom_node = jQuery(svg),
+		canvas = document.createElement('canvas'),
 		labels = $dom_node.next('.svg-graph-legend'),
 		$clone = $dom_node.clone(),
-		$container = $dom_node.closest('.dashbrd-grid-widget-content'),
+		$container = $dom_node.closest('.dashboard-grid-widget-content'),
 		image = new Image,
 		a = document.createElement('a'),
 		style = document.createElementNS('http://www.w3.org/1999/xhtml', 'style'),
@@ -850,7 +841,7 @@ function downloadSvgImage($dom_node, file_name) {
 	image.onload = function() {
 		context2d.drawImage(image, 0, 0);
 		a.href = canvas.toDataURL('image/png');
-		a.rel = 'noopener';
+		a.rel = 'noopener' + (ZBX_NOREFERER ? ' noreferrer' : '');
 		a.download = file_name;
 		a.target = '_blank';
 		a.click();
@@ -872,17 +863,138 @@ function downloadSvgImage($dom_node, file_name) {
 }
 
 /**
- * Download classic graph as .png image.
+ * Download classic image as given file name.
  *
- * @param {object} $dom_node    jQuery svg node to download.
- * @param {string} file_name    File name.
+ * @param {HTMLImageElement} img
+ * @param {string}           file_name
  */
-function downloadPngImage($dom_node, file_name) {
+function downloadPngImage(img, file_name) {
 	var a = document.createElement('a');
 
-	a.href = $dom_node.attr('src');
-	a.rel = 'noopener';
+	a.href = img.src;
+	a.rel = 'noopener' + (ZBX_NOREFERER ? ' noreferrer' : '');
 	a.download = file_name;
 	a.target = '_blank';
 	a.click();
 }
+
+/**
+ * Writes text into primary clipboard. Provides fallback for insecure context.
+ *
+ * @param {string} text  Text to write.
+ */
+function writeTextClipboard(text) {
+	if (window.isSecureContext) {
+		return window.navigator.clipboard.writeText(text);
+	}
+
+	const textarea = document.createElement('textarea');
+
+	textarea.value = text;
+	textarea.style.position = 'fixed';
+	document.body.appendChild(textarea);
+	textarea.select();
+	document.execCommand('copy');
+	textarea.remove();
+}
+
+function urlEncodeData(parameters, prefix = '') {
+	const result = [];
+
+	for (let [name, value] of Object.entries(parameters)) {
+		if (value === undefined) {
+			continue;
+		}
+
+		if (value === null) {
+			value = '';
+		}
+
+		const prefixed_name = prefix !== '' ? `${prefix}[${name}]` : name;
+
+		if (Array.isArray(value) || (typeof value === 'object')) {
+			const result_part = urlEncodeData(value, prefixed_name);
+
+			if (result_part !== '') {
+				result.push(result_part);
+			}
+		}
+		else {
+			result.push([encodeURIComponent(prefixed_name), encodeURIComponent(value)].join('='));
+		}
+	}
+
+	return result.join('&');
+}
+
+/**
+ * Get form field values as deep object.
+ *
+ * Example:
+ *     <form>
+ *         <input name="a" value="1">
+ *         <input name="b[c]" value="2">
+ *         <input name="b[d]" value="3">
+ *         <input name="e[f][]" value="4">
+ *         <input name="e[f][]" value="5">
+ *     </form>
+ *
+ *    ... will result in:
+ *
+ *    {
+ *        a: "1",
+ *        b: {
+ *            c: "2",
+ *            d: "3"
+ *        },
+ *        e: {
+ *            f: ["4", "5"]
+ *        }
+ *    }
+ *
+ * @param {HTMLFormElement} form
+ *
+ * @return {object}
+ */
+function getFormFields(form) {
+	const fields = {};
+
+	for (let [key, value] of new FormData(form)) {
+		value = value.replace(/\r?\n/g, '\r\n');
+
+		const key_parts = [...key.matchAll(/[^\[\]]+|\[\]/g)];
+
+		let key_fields = fields;
+
+		for (let i = 0; i < key_parts.length; i++) {
+			const key_part = key_parts[i][0];
+
+			if (i == key_parts.length - 1) {
+				if (key_part === '[]') {
+					key_fields.push(value);
+				}
+				else {
+					key_fields[key_part] = value;
+				}
+
+				break;
+			}
+
+			if (key_part === '[]') {
+				const key_field = key_parts[i + 1][0] === '[]' ? [] : {};
+
+				key_fields.push(key_field);
+				key_fields = key_field;
+			}
+			else {
+				if (!(key_part in key_fields)) {
+					key_fields[key_part] = key_parts[i + 1][0] === '[]' ? [] : {};
+				}
+
+				key_fields = key_fields[key_part];
+			}
+		}
+	}
+
+	return fields;
+}
