@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,24 +24,25 @@ require_once dirname(__FILE__).'/include/hosts.inc.php';
 
 $page['title'] = _('Availability report');
 $page['file'] = 'report2.php';
-$page['scripts'] = ['class.calendar.js', 'gtlc.js', 'multiselect.js'];
+$page['scripts'] = ['class.calendar.js', 'gtlc.js', 'report2.js'];
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'mode' =>			[T_ZBX_INT,			O_OPT,	P_SYS,			IN(implode(',', [AVAILABILITY_REPORT_BY_HOST, AVAILABILITY_REPORT_BY_TEMPLATE])),	null],
-	'hostgroupid' =>	[T_ZBX_INT,			O_OPT,	P_SYS,			DB_ID,		null],
-	'tpl_triggerid' =>	[T_ZBX_INT,			O_OPT,	P_SYS,			DB_ID,		null],
-	'triggerid' =>		[T_ZBX_INT,			O_OPT,	P_SYS|P_NZERO,	DB_ID,		null],
+	'mode' =>				[T_ZBX_INT,			O_OPT,	P_SYS,			IN(implode(',', [AVAILABILITY_REPORT_BY_HOST, AVAILABILITY_REPORT_BY_TEMPLATE])),	null],
+	'hostgroupid' =>		[T_ZBX_INT,			O_OPT,	P_SYS,			DB_ID,		null],
+	'tpl_triggerid' =>		[T_ZBX_INT,			O_OPT,	P_SYS,			DB_ID,		null],
+	'triggerid' =>			[T_ZBX_INT,			O_OPT,	P_SYS|P_NZERO,	DB_ID,		null],
 	// filter
-	'filter_groups' =>	[T_ZBX_INT,			O_OPT,	P_SYS,			DB_ID,		null],
-	'filter_hostids' =>	[T_ZBX_INT,			O_OPT,	P_SYS,			DB_ID,		null],
-	'filter_rst'=>		[T_ZBX_STR,			O_OPT,	P_SYS,			null,		null],
-	'filter_set' =>		[T_ZBX_STR,			O_OPT,	P_SYS,			null,		null],
-	'from' =>			[T_ZBX_RANGE_TIME,	O_OPT,	P_SYS,			null,		null],
-	'to' =>				[T_ZBX_RANGE_TIME,	O_OPT,	P_SYS,			null,		null],
+	'filter_groups' =>		[T_ZBX_INT,			O_OPT,	P_SYS,			DB_ID,		null],
+	'filter_hostids' =>		[T_ZBX_INT,			O_OPT,	P_SYS,			DB_ID,		null],
+	'filter_templateid' =>	[T_ZBX_INT,			O_OPT,	P_SYS,			DB_ID,		null],
+	'filter_rst'=>			[T_ZBX_STR,			O_OPT,	P_SYS,			null,		null],
+	'filter_set' =>			[T_ZBX_STR,			O_OPT,	P_SYS,			null,		null],
+	'from' =>				[T_ZBX_RANGE_TIME,	O_OPT,	P_SYS,			null,		null],
+	'to' =>					[T_ZBX_RANGE_TIME,	O_OPT,	P_SYS,			null,		null]
 ];
 check_fields($fields);
 validateTimeSelectorPeriod(getRequest('from'), getRequest('to'));
@@ -56,10 +57,10 @@ if ($report_mode == AVAILABILITY_REPORT_BY_TEMPLATE) {
 	if (getRequest('hostgroupid') && !isReadableHostGroups([getRequest('hostgroupid')])) {
 		access_deny();
 	}
-	if (getRequest('filter_groups') && !isReadableHostGroups([getRequest('filter_groups')])) {
+	if (getRequest('filter_groups') && !isReadableTemplateGroups([getRequest('filter_groups')])) {
 		access_deny();
 	}
-	if (getRequest('filter_hostids') && !isReadableTemplates([getRequest('filter_hostids')])) {
+	if (getRequest('filter_templateid') && !isReadableTemplates([getRequest('filter_templateid')])) {
 		access_deny();
 	}
 	if (getRequest('tpl_triggerid')) {
@@ -93,7 +94,7 @@ $key_prefix = 'web.avail_report.'.$report_mode;
 if (hasRequest('filter_set')) {
 	if ($report_mode == AVAILABILITY_REPORT_BY_TEMPLATE) {
 		CProfile::update($key_prefix.'.groupid', getRequest('filter_groups', 0), PROFILE_TYPE_ID);
-		CProfile::update($key_prefix.'.hostid', getRequest('filter_hostids', 0), PROFILE_TYPE_ID);
+		CProfile::update($key_prefix.'.hostid', getRequest('filter_templateid', 0), PROFILE_TYPE_ID);
 		CProfile::update($key_prefix.'.tpl_triggerid', getRequest('tpl_triggerid', 0), PROFILE_TYPE_ID);
 		CProfile::update($key_prefix.'.hostgroupid', getRequest('hostgroupid', 0), PROFILE_TYPE_ID);
 	}
@@ -121,7 +122,7 @@ $data['filter'] = ($report_mode == AVAILABILITY_REPORT_BY_TEMPLATE)
 		// 'Template group' field.
 		'groups' => getRequest('filter_groups', CProfile::get($key_prefix.'.groupid', 0)),
 		// 'Template' field.
-		'hostids' => getRequest('filter_hostids', CProfile::get($key_prefix.'.hostid', 0)),
+		'hostids' => getRequest('filter_templateid', CProfile::get($key_prefix.'.hostid', 0)),
 		// 'Template trigger' field.
 		'tpl_triggerid' => getRequest('tpl_triggerid', CProfile::get($key_prefix.'.tpl_triggerid', 0)),
 		// 'Host group' field.
@@ -143,8 +144,6 @@ $timeselector_options = [
 ];
 updateTimeSelectorPeriod($timeselector_options);
 
-$config = select_config();
-
 /*
  * Header
  */
@@ -157,7 +156,9 @@ $triggerData = isset($_REQUEST['triggerid'])
 	])
 	: null;
 
-$reportWidget = (new CWidget())->setTitle(_('Availability report'));
+$reportWidget = (new CWidget())
+	->setTitle(_('Availability report'))
+	->setDocUrl(CDocHelper::getUrl(CDocHelper::REPORT2));
 
 if ($triggerData) {
 	$triggerData = reset($triggerData);
@@ -187,19 +188,25 @@ else {
 	/**
 	 * Report list view (both data presentation modes).
 	 */
+
+	$select_mode = (new CSelect('mode'))
+		->setValue($report_mode)
+		->setFocusableElementId('mode')
+		->addOption(new CSelectOption(AVAILABILITY_REPORT_BY_HOST, _('By host')))
+		->addOption(new CSelectOption(AVAILABILITY_REPORT_BY_TEMPLATE, _('By trigger template')));
+
 	$reportWidget->setControls((new CForm('get'))
 		->cleanItems()
 		->setAttribute('aria-label', _('Main filter'))
 		->addItem((new CList())
 			->addItem([
-				new CLabel(_('Mode'), 'mode'),
+				new CLabel(_('Mode'), $select_mode->getFocusableElementId()),
 				(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-				new CComboBox('mode', $report_mode, 'submit()', [
-					AVAILABILITY_REPORT_BY_HOST => _('By host'),
-					AVAILABILITY_REPORT_BY_TEMPLATE => _('By trigger template')
-				])
+				$select_mode
 			])
-	));
+		)
+		->setName('report2')
+	);
 
 	/*
 	 * Filter
@@ -213,21 +220,23 @@ else {
 
 	// Make filter fields.
 	if ($report_mode == AVAILABILITY_REPORT_BY_TEMPLATE) {
-		// Sanitize $data['filter']['groups'] and prepare "Template group" combo box.
-		$groups = API::HostGroup()->get([
+		// Sanitize $data['filter']['groups'] and prepare "Template group" select options.
+		$groups = API::TemplateGroup()->get([
 			'output' => ['name'],
-			'templated_hosts' => true,
+			'with_templates' => true,
 			'with_triggers' => true,
 			'preservekeys' => true
 		]);
-		$groups = enrichParentGroups($groups);
+
+		$groups = enrichParentTemplateGroups($groups);
+
 		CArrayHelper::sort($groups, ['name']);
 
 		if (!array_key_exists($data['filter']['groups'], $groups)) {
 			$data['filter']['groups'] = 0;
 		}
 
-		// Sanitize $data['filter']['hostids'] and prepare "Template" combo box.
+		// Sanitize $data['filter']['hostids'] and prepare "Template" select options.
 		$templates = API::Template()->get([
 			'output' => ['name'],
 			'groupids' => $data['filter']['groups'] ? [$data['filter']['groups']] : null,
@@ -240,14 +249,16 @@ else {
 			$data['filter']['hostids'] = 0;
 		}
 
-		$filter_hostid_combobox = (new CComboBox('filter_hostids', $data['filter']['hostids'], 'javascript: submit();'))
-			->addItem(0, _('all'));
+		$select_filter_hostid = (new CSelect('filter_templateid'))
+			->setValue($data['filter']['hostids'])
+			->setFocusableElementId('filter-templateid')
+			->addOption(new CSelectOption(0, _('all')));
 
 		foreach ($templates as $templateid => $template) {
-			$filter_hostid_combobox->addItem($templateid, $template['name']);
+			$select_filter_hostid->addOption(new CSelectOption($templateid, $template['name']));
 		}
 
-		// Sanitize $data['filter']['tpl_triggerid'] and prepare "Template Trigger" combo box.
+		// Sanitize $data['filter']['tpl_triggerid'] and prepare "Template Trigger" select options.
 		$triggers = API::Trigger()->get([
 			'output' => ['description'],
 			'selectHosts' => ['name'],
@@ -281,23 +292,24 @@ else {
 			$data['filter']['tpl_triggerid'] = 0;
 		}
 
-		$tpl_triggerid_combobox = (new CComboBox('tpl_triggerid', $data['filter']['tpl_triggerid'], 'javascript: submit()'))
-			->addItem(0, _('all'));
+		$select_tpl_triggerid = (new CSelect('tpl_triggerid'))
+			->setValue($data['filter']['tpl_triggerid'])
+			->setFocusableElementId('tpl-triggerid')
+			->addOption(new CSelectOption(0, _('all')));
 
 		$tpl_triggerids = [];
 
 		foreach ($triggers as $triggerid => $trigger) {
-			$tpl_triggerid_combobox->addItem($triggerid,
-				(($data['filter']['hostids'] == 0) ? $trigger['hosts'][0]['name'].NAME_DELIMITER : '').$trigger['description']
-			);
+			$label = (($data['filter']['hostids'] == 0) ? $trigger['hosts'][0]['name'].NAME_DELIMITER : '').$trigger['description'];
+			$select_tpl_triggerid->addOption(new CSelectOption($triggerid, $label));
 
 			$tpl_triggerids[$triggerid] = true;
 		}
 
-		// Sanitize $data['filter']['hostgroupid'] and prepare "Host Group" combo box.
+		// Sanitize $data['filter']['hostgroupid'] and prepare "Host Group" select options.
 		$host_groups = API::HostGroup()->get([
 			'output' => ['name'],
-			'monitored_hosts' => true,
+			'with_monitored_hosts' => true,
 			'preservekeys' => true
 		]);
 		$host_groups = enrichParentGroups($host_groups);
@@ -307,11 +319,13 @@ else {
 			$data['filter']['hostgroupid'] = 0;
 		}
 
-		$hostgroupid_combobox = (new CComboBox('hostgroupid', $data['filter']['hostgroupid'], 'javascript: submit()'))
-			->addItem(0, _('all'));
+		$select_hostgroupid = (new CSelect('hostgroupid'))
+			->setValue($data['filter']['hostgroupid'])
+			->setFocusableElementId('hostgroupid')
+			->addOption(new CSelectOption(0, _('all')));
 
 		foreach ($host_groups as $groupid => $group) {
-			$hostgroupid_combobox->addItem($groupid, $group['name']);
+			$select_hostgroupid->addOption(new CSelectOption($groupid, $group['name']));
 		}
 
 		$hostgroupids = [];
@@ -345,6 +359,7 @@ else {
 
 		if ($templated_triggers_all) {
 			// Select monitored host triggers, derived from templates and belonging to the requested groups.
+			$limit = CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1;
 			$triggers = API::Trigger()->get([
 				'output' => ['triggerid', 'description', 'expression', 'value'],
 				'selectHosts' => ['name'],
@@ -352,7 +367,7 @@ else {
 				'monitored' => true,
 				'groupids' => ($data['filter']['hostgroupid'] == 0) ? null : array_keys($hostgroupids),
 				'filter' => ['templateid' => array_keys($templated_triggers_all)],
-				'limit' => $config['search_limit'] + 1
+				'limit' => $limit
 			]);
 		}
 		else {
@@ -360,19 +375,33 @@ else {
 			$triggers = [];
 		}
 
-		$filter_groupid_combobox = (new CComboBox('filter_groups', $data['filter']['groups'], 'javascript: submit();'))
+		$select_filter_groupid = (new CSelect('filter_groups'))
 			->setAttribute('autofocus', 'autofocus')
-			->addItem(0, _('all'));
+			->setValue($data['filter']['groups'])
+			->setFocusableElementId('filter-groups')
+			->addOption(new CSelectOption(0, _('all')));
 
 		foreach ($groups as $groupid => $group) {
-			$filter_groupid_combobox->addItem($groupid, $group['name']);
+			$select_filter_groupid->addOption(new CSelectOption($groupid, $group['name']));
 		}
 
 		$filter_column
-			->addRow(_('Template group'), $filter_groupid_combobox)
-			->addRow(_('Template'), $filter_hostid_combobox)
-			->addRow(_('Template trigger'), $tpl_triggerid_combobox)
-			->addRow(_('Host group'), $hostgroupid_combobox)
+			->addRow(
+				new CLabel(_('Template group'), $select_filter_groupid->getFocusableElementId()),
+				$select_filter_groupid
+			)
+			->addRow(
+				new CLabel(_('Template'), $select_filter_hostid->getFocusableElementId()),
+				$select_filter_hostid
+			)
+			->addRow(
+				new CLabel(_('Template trigger'), $select_tpl_triggerid->getFocusableElementId()),
+				$select_tpl_triggerid
+			)
+			->addRow(
+				new CLabel(_('Host group'), $select_hostgroupid->getFocusableElementId()),
+				$select_hostgroupid
+			)
 			->addVar('filter_set', '1');
 	}
 	// Report by host.
@@ -382,8 +411,7 @@ else {
 			? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
 				'output' => ['groupid', 'name'],
 				'groupids' => $data['filter']['groups'],
-				'with_triggers' => true,
-				'monitored_hosts' => true,
+				'with_monitored_hosts' => true,
 				'preservekeys' => true
 			]), ['groupid' => 'id'])
 			: [];
@@ -406,6 +434,7 @@ else {
 		// Select monitored host triggers, derived from templates and belonging to the requested groups.
 		$groups = enrichParentGroups($data['filter']['groups']);
 
+		$limit = CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1;
 		$triggers = API::Trigger()->get([
 			'output' => ['triggerid', 'description', 'expression', 'value'],
 			'selectHosts' => ['name'],
@@ -413,7 +442,7 @@ else {
 			'hostids' => $data['filter']['hostids'] ? array_keys($data['filter']['hostids']) : null,
 			'expandDescription' => true,
 			'monitored' => true,
-			'limit' => $config['search_limit'] + 1
+			'limit' => $limit
 		]);
 
 		$filter_column
@@ -430,7 +459,7 @@ else {
 							'dstfrm' => 'zbx_filter',
 							'dstfld1' => 'filter_groups_',
 							'with_triggers' => true,
-							'real_hosts' => 1
+							'enrich_parent_groups' => true
 						]
 					]
 				]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
@@ -465,7 +494,8 @@ else {
 	unset($trigger);
 
 	$reportWidget->addItem(
-		(new CFilter(new CUrl('report2.php')))
+		(new CFilter())
+			->setResetUrl(new CUrl('report2.php'))
 			->setProfile($data['filter']['timeline']['profileIdx'])
 			->setActiveTab($data['filter']['active_tab'])
 			->addFormItem((new CVar('mode', $report_mode))->removeId())
@@ -485,31 +515,38 @@ else {
 	$page_num = getRequest('page', 1);
 	CPagerHelper::savePage($page['file'], $page_num);
 	$paging = CPagerHelper::paginate($page_num, $triggers, ZBX_SORT_UP, new CUrl('report2.php'));
+	$allowed_ui_problems = CWebUser::checkAccess(CRoleHelper::UI_MONITORING_PROBLEMS);
 
 	foreach ($triggers as $trigger) {
 		$availability = calculateAvailability($trigger['triggerid'], $data['filter']['timeline']['from_ts'],
 			$data['filter']['timeline']['to_ts']
 		);
 
+		$url = (new CUrl('report2.php'))->setArgument('triggerid', $trigger['triggerid']);
+		if ($report_mode == AVAILABILITY_REPORT_BY_TEMPLATE) {
+			$url->setArgument('filter_templateid', $data['filter']['hostids']);
+		}
+		else {
+			$url->setArgument('filter_hostids', $trigger['hosts'][0]['hostid']);
+		}
+
 		$triggerTable->addRow([
 			$trigger['host_name'],
-			new CLink($trigger['description'],
-				(new CUrl('zabbix.php'))
-					->setArgument('action', 'problem.view')
-					->setArgument('filter_triggerids', [$trigger['triggerid']])
-					->setArgument('filter_set', '1')
-			),
+			$allowed_ui_problems
+				? new CLink($trigger['description'],
+					(new CUrl('zabbix.php'))
+						->setArgument('action', 'problem.view')
+						->setArgument('filter_name', '')
+						->setArgument('triggerids', [$trigger['triggerid']])
+				)
+				: $trigger['description'],
 			($availability['true'] < 0.00005)
 				? ''
 				: (new CSpan(sprintf('%.4f%%', $availability['true'])))->addClass(ZBX_STYLE_RED),
 			($availability['false'] < 0.00005)
 				? ''
 				: (new CSpan(sprintf('%.4f%%', $availability['false'])))->addClass(ZBX_STYLE_GREEN),
-			new CLink(_('Show'),
-				(new CUrl('report2.php'))
-					->setArgument('filter_hostids', $trigger['hosts'][0]['hostid'])
-					->setArgument('triggerid', $trigger['triggerid'])
-			)
+			new CLink(_('Show'), $url)
 		]);
 	}
 
@@ -518,8 +555,7 @@ else {
 		'domid' => 'avail_report',
 		'loadSBox' => 0,
 		'loadImage' => 0,
-		'dynamic' => 0,
-		'mainObject' => 1
+		'dynamic' => 0
 	];
 	zbx_add_post_js(
 		'timeControl.addObject("avail_report", '.zbx_jsvalue($data['filter']).', '.zbx_jsvalue($obj_data).');'
